@@ -102,6 +102,8 @@ def main():
         main_win32()
     elif sys.platform.startswith('linux'):
         main_posix('linux', '.so')
+    elif sys.platform.startswith('freebsd'):
+        main_posix('freebsd', '.so')
     elif sys.platform == 'darwin':
         main_posix('osx', '.dylib')
     else:
