@@ -29,6 +29,29 @@ def _binop(opname, cls=instructions.Instruction):
     return wrap
 
 
+def _binop_with_overflow(opname, cls=instructions.Instruction):
+    def wrap(fn):
+        @functools.wraps(fn)
+        def wrapped(self, lhs, rhs, name=''):
+            assert lhs.type == rhs.type, "Operands must be the same type"
+            ty = lhs.type
+            if not isinstance(ty, types.IntType):
+                raise TypeError("expected an integer type, got %s" % (ty,))
+            bool_ty = types.IntType(1)
+
+            mod = self.module
+            fnty = types.FunctionType(types.LiteralStructType([ty, bool_ty]),
+                                      [ty, ty])
+            fn = mod.declare_intrinsic("llvm.%s.with.overflow" % (opname,),
+                                       [ty], fnty)
+            ret = self.call(fn, [lhs, rhs], name=name)
+            return ret
+
+        return wrapped
+
+    return wrap
+
+
 def _uniop(opname, cls=instructions.Instruction):
     def wrap(fn):
         @functools.wraps(fn)
@@ -268,6 +291,30 @@ class IRBuilder(object):
     def xor(self, lhs, rhs, name=''):
         pass
 
+    @_binop_with_overflow('sadd')
+    def sadd_with_overflow(self, lhs, rhs, name=''):
+        pass
+
+    @_binop_with_overflow('smul')
+    def smul_with_overflow(self, lhs, rhs, name=''):
+        pass
+
+    @_binop_with_overflow('ssub')
+    def ssub_with_overflow(self, lhs, rhs, name=''):
+        pass
+
+    @_binop_with_overflow('uadd')
+    def uadd_with_overflow(self, lhs, rhs, name=''):
+        pass
+
+    @_binop_with_overflow('umul')
+    def umul_with_overflow(self, lhs, rhs, name=''):
+        pass
+
+    @_binop_with_overflow('usub')
+    def usub_with_overflow(self, lhs, rhs, name=''):
+        pass
+
     #
     # Unary APIs
     #
@@ -437,6 +484,13 @@ class IRBuilder(object):
         self._set_terminator(br)
         return br
 
+    def branch_indirect(self, addr):
+        """Branch indirectly
+        """
+        br = instructions.IndirectBranch(self.block, "indirectbr", addr)
+        self._set_terminator(br)
+        return br
+
     def ret_void(self):
         return self._set_terminator(
             instructions.Ret(self.block, "ret void"))
@@ -445,6 +499,13 @@ class IRBuilder(object):
         return self._set_terminator(
             instructions.Ret(self.block, "ret", value))
 
+    def resume(self, landingpad):
+        """Resume an in-flight exception
+        """
+        br = instructions.Branch(self.block, "resume", [landingpad])
+        self._set_terminator(br)
+        return br
+
     # Call APIs
 
     def call(self, fn, args, name='', cconv=None, tail=False):
@@ -453,6 +514,12 @@ class IRBuilder(object):
         self._insert(inst)
         return inst
 
+    def invoke(self, fn, args, normal_to, unwind_to, name='', cconv=None, tail=False):
+        inst = instructions.InvokeInstr(self.block, fn, args, normal_to, unwind_to, name=name,
+                                        cconv=cconv)
+        self._set_terminator(inst)
+        return inst
+
     # GEP APIs
 
     def gep(self, ptr, indices, inbounds=False, name=''):
@@ -489,6 +556,7 @@ class IRBuilder(object):
     def unreachable(self):
         inst = instructions.Unreachable(self.block)
         self._set_terminator(inst)
+        return inst
 
     def atomic_rmw(self, op, ptr, val, ordering, name=''):
         inst = instructions.AtomicRMW(self.block, op, ptr, val, ordering, name=name)
@@ -505,6 +573,11 @@ class IRBuilder(object):
         self._insert(inst)
         return inst
 
+    def landingpad(self, typ, personality, name='', cleanup=False):
+        inst = instructions.LandingPadInstr(self.block, typ, personality, name, cleanup)
+        self._insert(inst)
+        return inst
+
     def assume(self, cond):
         fn = self.module.declare_intrinsic("llvm.assume")
         return self.call(fn, [cond])
