@@ -7,6 +7,7 @@ import subprocess
 import sys
 import unittest
 import platform
+import locale
 
 from llvmlite import six
 from llvmlite import binding as llvm
@@ -14,6 +15,18 @@ from llvmlite.binding import ffi
 from . import TestCase
 
 
+def no_de_locale():
+    cur = locale.setlocale(locale.LC_ALL)
+    try:
+        locale.setlocale(locale.LC_ALL, 'de_DE')
+    except locale.Error:
+        return True
+    else:
+        return False
+    finally:
+        locale.setlocale(locale.LC_ALL, cur)
+
+
 asm_sum = r"""
     ; ModuleID = '<string>'
     target triple = "{triple}"
@@ -79,6 +92,17 @@ asm_sum_declare = r"""
     declare i32 @sum(i32 %.1, i32 %.2)
     """
 
+
+asm_double_locale = r"""
+    ; ModuleID = '<string>'
+    target triple = "{triple}"
+
+    define void @foo() {{
+      %const = fadd double 0.0, 3.14
+      ret void
+    }}
+    """
+
 class BaseTest(TestCase):
 
     def setUp(self):
@@ -160,6 +184,21 @@ class TestMisc(BaseTest):
         self.assertIn((major, minor), [(3, 5), (3, 6)])
         self.assertIn(patch, range(10))
 
+    def test_check_jit_execution(self):
+        llvm.check_jit_execution()
+
+    @unittest.skipIf(no_de_locale(), "Locale not available")
+    def test_print_double_locale(self):
+        m = self.module(asm_double_locale)
+        expect = str(m)
+        # Change the locale so that comma is used as decimal-point
+        # to trigger the LLVM bug (llvmlite issue #80)
+        locale.setlocale(locale.LC_ALL, 'de_DE')
+        # The LLVM bug is trigged by print the module with double constant
+        got = str(m)
+        # Changing the locale should not affect the LLVM IR
+        self.assertEqual(expect, got)
+
 
 class TestModuleRef(BaseTest):
 
@@ -187,6 +226,13 @@ class TestModuleRef(BaseTest):
             with mod:
                 pass
 
+    def test_name(self):
+        mod = self.module()
+        mod.name = "foo"
+        self.assertEqual(mod.name, "foo")
+        mod.name = "bar"
+        self.assertEqual(mod.name, "bar")
+
     def test_data_layout(self):
         mod = self.module()
         s = mod.data_layout
@@ -318,13 +364,16 @@ class JITTestMixin(object):
     Mixin for ExecutionEngine tests.
     """
 
+    def get_sum(self, ee, func_name="sum"):
+        ee.finalize_object()
+        cfptr = ee.get_function_address(func_name)
+        self.assertTrue(cfptr)
+        return CFUNCTYPE(c_int, c_int, c_int)(cfptr)
+
     def test_run_code(self):
         mod = self.module()
         with self.jit(mod) as ee:
-            ee.finalize_object()
-            cfptr = ee.get_pointer_to_global(mod.get_function('sum'))
-
-            cfunc = CFUNCTYPE(c_int, c_int, c_int)(cfptr)
+            cfunc = self.get_sum(ee)
             res = cfunc(2, -5)
             self.assertEqual(-3, res)
 
@@ -423,6 +472,77 @@ class JITTestMixin(object):
         self.assertEqual(td.get_pointee_abi_size(gv_struct.type), 24)
         self.assertIn(td.get_pointee_abi_alignment(gv_struct.type), (4, 8))
 
+    def test_object_cache_notify(self):
+        notifies = []
+
+        def notify(mod, buf):
+            notifies.append((mod, buf))
+
+        mod = self.module()
+        ee = self.jit(mod)
+        ee.set_object_cache(notify)
+
+        self.assertEqual(len(notifies), 0)
+        cfunc = self.get_sum(ee)
+        cfunc(2, -5)
+        self.assertEqual(len(notifies), 1)
+        # The right module object was found
+        self.assertIs(notifies[0][0], mod)
+        self.assertIsInstance(notifies[0][1], bytes)
+
+        notifies[:] = []
+        mod2 = self.module(asm_mul)
+        ee.add_module(mod2)
+        cfunc = self.get_sum(ee, "mul")
+        self.assertEqual(len(notifies), 1)
+        # The right module object was found
+        self.assertIs(notifies[0][0], mod2)
+        self.assertIsInstance(notifies[0][1], bytes)
+
+    def test_object_cache_getbuffer(self):
+        notifies = []
+        getbuffers = []
+
+        def notify(mod, buf):
+            notifies.append((mod, buf))
+
+        def getbuffer(mod):
+            getbuffers.append(mod)
+
+        mod = self.module()
+        ee = self.jit(mod)
+        ee.set_object_cache(notify, getbuffer)
+
+        # First return None from getbuffer(): the object is compiled normally
+        self.assertEqual(len(notifies), 0)
+        self.assertEqual(len(getbuffers), 0)
+        cfunc = self.get_sum(ee)
+        self.assertEqual(len(notifies), 1)
+        self.assertEqual(len(getbuffers), 1)
+        self.assertIs(getbuffers[0], mod)
+        sum_buffer = notifies[0][1]
+
+        # Recreate a new EE, and use getbuffer() to return the previously
+        # compiled object.
+
+        def getbuffer_successful(mod):
+            getbuffers.append(mod)
+            return sum_buffer
+
+        notifies[:] = []
+        getbuffers[:] = []
+        # Use another source module to make sure it is ignored
+        mod = self.module(asm_mul)
+        ee = self.jit(mod)
+        ee.set_object_cache(notify, getbuffer_successful)
+
+        self.assertEqual(len(notifies), 0)
+        self.assertEqual(len(getbuffers), 0)
+        cfunc = self.get_sum(ee)
+        self.assertEqual(cfunc(2, -5), -3)
+        self.assertEqual(len(notifies), 0)
+        self.assertEqual(len(getbuffers), 1)
+
 
 class JITWithTMTestMixin(JITTestMixin):
 
@@ -750,7 +870,7 @@ class TestDylib(BaseTest):
         with self.assertRaises(RuntimeError):
             llvm.load_library_permanently("zzzasdkf;jasd;l")
 
-    @unittest.skipUnless(platform.system() in ["Linux", "Darwin"], 
+    @unittest.skipUnless(platform.system() in ["Linux", "Darwin"],
                          "test only works on Linux and Darwin")
     def test_libm(self):
         system = platform.system()
